@@ -7,6 +7,7 @@ from lightly.api.api_workflow_tags import _TagsMixin
 from requests import Response
 
 from lightly.__init__ import __version__
+from lightly.api.api_workflow_compute_worker import _ComputeWorkerMixin
 from lightly.api.api_workflow_datasets import _DatasetsMixin
 from lightly.api.api_workflow_datasources import _DatasourcesMixin
 from lightly.api.api_workflow_download_dataset import _DownloadDatasetMixin
@@ -25,6 +26,7 @@ from lightly.openapi_generated.swagger_client.api.datasets_api import \
     DatasetsApi
 from lightly.openapi_generated.swagger_client.api.datasources_api import \
     DatasourcesApi
+from lightly.openapi_generated.swagger_client.api.docker_api import DockerApi
 from lightly.openapi_generated.swagger_client.api.embeddings_api import \
     EmbeddingsApi
 from lightly.openapi_generated.swagger_client.api.jobs_api import JobsApi
@@ -49,7 +51,8 @@ class ApiWorkflowClient(_UploadEmbeddingsMixin,
                         _DatasetsMixin,
                         _UploadCustomMetadataMixin,
                         _TagsMixin,
-                        _DatasourcesMixin
+                        _DatasourcesMixin,
+                        _ComputeWorkerMixin,
                         ):
     """Provides a uniform interface to communicate with the api 
     
@@ -70,12 +73,18 @@ class ApiWorkflowClient(_UploadEmbeddingsMixin,
             but used by a workflow, the newest embedding is taken by default
     """
 
-    def __init__(self, token: str, dataset_id: str = None, embedding_id: str = None):
+    def __init__(self, token: str = None, dataset_id: str = None, embedding_id: str = None):
 
         self.check_version_compatibility()
 
         configuration = Configuration()
         configuration.host = getenv('LIGHTLY_SERVER_LOCATION', 'https://api.lightly.ai')
+        if token is None:
+            token = getenv('LIGHTLY_TOKEN', None)
+            if token is None:
+                raise ValueError(f"Either provide a 'token' argument or export"
+                                 f" a LIGHTLY_TOKEN environment variable")
+                
         configuration.api_key = {'token': token}
         api_client = ApiClient(configuration=configuration)
         self.api_client = api_client
@@ -86,6 +95,7 @@ class ApiWorkflowClient(_UploadEmbeddingsMixin,
         if embedding_id is not None:
             self.embedding_id = embedding_id
 
+        self._compute_worker_api = DockerApi(api_client=self.api_client)
         self._datasets_api = DatasetsApi(api_client=self.api_client)
         self._datasources_api = DatasourcesApi(api_client=self.api_client)
         self._samplings_api = SamplingsApi(api_client=self.api_client)
@@ -114,7 +124,7 @@ class ApiWorkflowClient(_UploadEmbeddingsMixin,
         try:
             return self._dataset_id
         except AttributeError:
-            all_datasets: List[DatasetData] = self._datasets_api.get_datasets()
+            all_datasets: List[DatasetData] = self.get_datasets()
             datasets_sorted = sorted(all_datasets, key=lambda dataset: dataset.last_modified_at)
             last_modified_dataset = datasets_sorted[-1]
             self._dataset_id = last_modified_dataset.id
@@ -122,6 +132,25 @@ class ApiWorkflowClient(_UploadEmbeddingsMixin,
                                       f"taking the last modified dataset {last_modified_dataset.name} as default dataset."))
             return self._dataset_id
 
+    @dataset_id.setter
+    def dataset_id(self, dataset_id: str):
+        """Sets the current dataset id for the client.
+        
+        Args:
+            dataset_id:
+                The new dataset id.
+
+        Raises:
+            ValueError if the dataset id does not exist.
+        """
+        if not self.dataset_exists(dataset_id):
+            raise ValueError(
+                f"A dataset with the id {dataset_id} does not exist on the web"
+                f"platform."
+            )
+        self._dataset_id = dataset_id
+        
+
     def _order_list_by_filenames(
             self, filenames_for_list: List[str],
             list_to_order: List[object]
